"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var state_kernel_1 = require("../state-kernel");
var routing_actions_1 = require("../redux/actions/routing-actions");
var history_reconciler_1 = require("./routing/history-reconciler");
var RoutingKernel = /** @class */ (function (_super) {
    __extends(RoutingKernel, _super);
    function RoutingKernel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    RoutingKernel.prototype.getState = function (fullState) {
        return fullState.routing;
    };
    RoutingKernel.prototype.handleStateChange = function (state) {
        var _this = this;
        Object.keys(state.history).forEach(function (viewName) {
            _this.reconcileHistories(viewName, state.history[viewName]);
        });
    };
    RoutingKernel.prototype.onFramework7Set = function () {
        var _this = this;
        this.framework7.once('pageInit', function () { return _this.initializeHistory(); });
    };
    RoutingKernel.prototype.initializeHistory = function () {
        var _this = this;
        var mainView = this.framework7.views.find(function (view) { return view.main; });
        if (!mainView) {
            throw new Error('Framework7 Redux requires a main view');
        }
        this.framework7.views.forEach(function (view) {
            var router = view.router;
            if (router.history.length) {
                router.history.forEach(function (f7HistoryUrl) {
                    _this.dispatchAction(routing_actions_1.navigateTo(f7HistoryUrl, false, view.name || view.main && 'main'));
                });
            }
        });
    };
    RoutingKernel.prototype.reconcileHistories = function (viewName, newHistory) {
        if (!this.framework7)
            return;
        var view = this.framework7.views.find(function (view) {
            if (viewName === 'main') {
                return view.main;
            }
            else {
                return view.name === viewName;
            }
        });
        if (!view) {
            // If view no longer exists, ignore it for now. The state for that view will get preserved in the store
            // and if that view gets recreated at some point, it will be restored to the correct state. For example,
            // if a page is using a routable panel that has a view with multiple pages, and that routable panel gets
            // destroyed after moving elsewhere, and then gets recreated after navigating back to the original page,
            // this will preserve the page in the panel where the user left off.
            return;
        }
        var reconciler = new history_reconciler_1.HistoryReconciler(view.router, view.router.history, newHistory);
        reconciler
            .getOperationsToReconcileHistories()
            .forEach(function (operation) {
            if (operation.forward) {
                view.router.navigate(operation.url, { reloadCurrent: operation.replace });
            }
            else {
                view.router.back();
            }
        });
    };
    return RoutingKernel;
}(state_kernel_1.StateKernel));
exports.RoutingKernel = RoutingKernel;
